# Copyright 2016- Game Server Services, Inc. or its affiliates. All Rights
# Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License").
# You may not use this file except in compliance with the License.
# A copy of the License is located at
#
#  http://www.apache.org/licenses/LICENSE-2.0
#
# or in the "license" file accompanying this file. This file is distributed
# on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either
# express or implied. See the License for the specific language governing
# permissions and limitations under the License.
from __future__ import annotations
from typing import *
from .Version import Version
from .ScheduleVersion import ScheduleVersion
from .options.VersionModelOptions import VersionModelOptions
from .options.VersionModelTypeIsSimpleOptions import VersionModelTypeIsSimpleOptions
from .options.VersionModelTypeIsScheduleOptions import VersionModelTypeIsScheduleOptions
from .options.VersionModelScopeIsPassiveOptions import VersionModelScopeIsPassiveOptions
from .options.VersionModelScopeIsActiveOptions import VersionModelScopeIsActiveOptions
from .enum.VersionModelScope import VersionModelScope
from .enum.VersionModelType import VersionModelType


class VersionModel:
    name: str
    scope: VersionModelScope
    type: VersionModelType
    metadata: Optional[str] = None
    current_version: Optional[Version] = None
    warning_version: Optional[Version] = None
    error_version: Optional[Version] = None
    schedule_versions: Optional[List[ScheduleVersion]] = None
    need_signature: Optional[bool] = None
    signature_key_id: Optional[str] = None

    def __init__(
        self,
        name: str,
        scope: VersionModelScope,
        type: VersionModelType,
        options: Optional[VersionModelOptions] = VersionModelOptions(),
    ):
        self.name = name
        self.scope = scope
        self.type = type
        self.metadata = options.metadata if options.metadata else None
        self.current_version = options.current_version if options.current_version else None
        self.warning_version = options.warning_version if options.warning_version else None
        self.error_version = options.error_version if options.error_version else None
        self.schedule_versions = options.schedule_versions if options.schedule_versions else None
        self.need_signature = options.need_signature if options.need_signature else None
        self.signature_key_id = options.signature_key_id if options.signature_key_id else None

    @staticmethod
    def type_is_simple(
        name: str,
        scope: VersionModelScope,
        warning_version: Version,
        error_version: Version,
        options: Optional[VersionModelTypeIsSimpleOptions] = VersionModelTypeIsSimpleOptions(),
    ) -> VersionModel:
        return VersionModel(
            name,
            scope,
            VersionModelType.SIMPLE,
            VersionModelOptions(
                warning_version,
                error_version,
                options.metadata,
                options.schedule_versions,
            ),
        )

    @staticmethod
    def type_is_schedule(
        name: str,
        scope: VersionModelScope,
        options: Optional[VersionModelTypeIsScheduleOptions] = VersionModelTypeIsScheduleOptions(),
    ) -> VersionModel:
        return VersionModel(
            name,
            scope,
            VersionModelType.SCHEDULE,
            VersionModelOptions(
                options.metadata,
                options.schedule_versions,
            ),
        )

    @staticmethod
    def scope_is_passive(
        name: str,
        type: VersionModelType,
        need_signature: bool,
        options: Optional[VersionModelScopeIsPassiveOptions] = VersionModelScopeIsPassiveOptions(),
    ) -> VersionModel:
        return VersionModel(
            name,
            VersionModelScope.PASSIVE,
            type,
            VersionModelOptions(
                need_signature,
                options.metadata,
                options.schedule_versions,
            ),
        )

    @staticmethod
    def scope_is_active(
        name: str,
        type: VersionModelType,
        options: Optional[VersionModelScopeIsActiveOptions] = VersionModelScopeIsActiveOptions(),
    ) -> VersionModel:
        return VersionModel(
            name,
            VersionModelScope.ACTIVE,
            type,
            VersionModelOptions(
                options.metadata,
                options.schedule_versions,
            ),
        )

    def properties(
        self,
    ) -> Dict[str, Any]:
        properties: Dict[str, Any] = {}

        if self.name is not None:
            properties["name"] = self.name
        if self.metadata is not None:
            properties["metadata"] = self.metadata
        if self.scope is not None:
            properties["scope"] = self.scope.value
        if self.type is not None:
            properties["type"] = self.type.value
        if self.current_version is not None:
            properties["currentVersion"] = self.current_version.properties(
            )
        if self.warning_version is not None:
            properties["warningVersion"] = self.warning_version.properties(
            )
        if self.error_version is not None:
            properties["errorVersion"] = self.error_version.properties(
            )
        if self.schedule_versions is not None:
            properties["scheduleVersions"] = [
                v.properties(
                )
                for v in self.schedule_versions
            ]
        if self.need_signature is not None:
            properties["needSignature"] = self.need_signature
        if self.signature_key_id is not None:
            properties["signatureKeyId"] = self.signature_key_id

        return properties



# Copyright 2016- Game Server Services, Inc. or its affiliates. All Rights
# Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License").
# You may not use this file except in compliance with the License.
# A copy of the License is located at
#
#  http://www.apache.org/licenses/LICENSE-2.0
#
# or in the "license" file accompanying this file. This file is distributed
# on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either
# express or implied. See the License for the specific language governing
# permissions and limitations under the License.
from __future__ import annotations
from typing import *
from .ChangeStateEvent import ChangeStateEvent
from .EmitEvent import EmitEvent
from .options.EventOptions import EventOptions
from .options.EventEventTypeIsChangeStateOptions import EventEventTypeIsChangeStateOptions
from .options.EventEventTypeIsEmitOptions import EventEventTypeIsEmitOptions
from .enum.EventEventType import EventEventType


class Event:
    event_type: EventEventType
    change_state_event: Optional[ChangeStateEvent] = None
    emit_event: Optional[EmitEvent] = None

    def __init__(
        self,
        event_type: EventEventType,
        options: Optional[EventOptions] = EventOptions(),
    ):
        self.event_type = event_type
        self.change_state_event = options.change_state_event if options.change_state_event else None
        self.emit_event = options.emit_event if options.emit_event else None

    @staticmethod
    def event_type_is_change_state(
        change_state_event: ChangeStateEvent,
        options: Optional[EventEventTypeIsChangeStateOptions] = EventEventTypeIsChangeStateOptions(),
    ) -> Event:
        return Event(
            EventEventType.CHANGE_STATE,
            EventOptions(
                change_state_event,
            ),
        )

    @staticmethod
    def event_type_is_emit(
        emit_event: EmitEvent,
        options: Optional[EventEventTypeIsEmitOptions] = EventEventTypeIsEmitOptions(),
    ) -> Event:
        return Event(
            EventEventType.EMIT,
            EventOptions(
                emit_event,
            ),
        )

    def properties(
        self,
    ) -> Dict[str, Any]:
        properties: Dict[str, Any] = {}

        if self.event_type is not None:
            properties["eventType"] = self.event_type.value
        if self.change_state_event is not None:
            properties["changeStateEvent"] = self.change_state_event.properties(
            )
        if self.emit_event is not None:
            properties["emitEvent"] = self.emit_event.properties(
            )

        return properties



# Copyright 2016- Game Server Services, Inc. or its affiliates. All Rights
# Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License").
# You may not use this file except in compliance with the License.
# A copy of the License is located at
#
#  http://www.apache.org/licenses/LICENSE-2.0
#
# or in the "license" file accompanying this file. This file is distributed
# on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either
# express or implied. See the License for the specific language governing
# permissions and limitations under the License.
from __future__ import annotations
from typing import *
from .RoleModel import RoleModel
from .options.GuildModelOptions import GuildModelOptions


class GuildModel:
    name: str
    default_maximum_member_count: int
    maximum_member_count: int
    roles: List[RoleModel]
    guild_master_role: str
    guild_member_default_role: str
    rejoin_cool_time_minutes: int
    metadata: Optional[str] = None

    def __init__(
        self,
        name: str,
        default_maximum_member_count: int,
        maximum_member_count: int,
        roles: List[RoleModel],
        guild_master_role: str,
        guild_member_default_role: str,
        rejoin_cool_time_minutes: int,
        options: Optional[GuildModelOptions] = GuildModelOptions(),
    ):
        self.name = name
        self.default_maximum_member_count = default_maximum_member_count
        self.maximum_member_count = maximum_member_count
        self.roles = roles
        self.guild_master_role = guild_master_role
        self.guild_member_default_role = guild_member_default_role
        self.rejoin_cool_time_minutes = rejoin_cool_time_minutes
        self.metadata = options.metadata if options.metadata else None

    def properties(
        self,
    ) -> Dict[str, Any]:
        properties: Dict[str, Any] = {}

        if self.name is not None:
            properties["name"] = self.name
        if self.metadata is not None:
            properties["metadata"] = self.metadata
        if self.default_maximum_member_count is not None:
            properties["defaultMaximumMemberCount"] = self.default_maximum_member_count
        if self.maximum_member_count is not None:
            properties["maximumMemberCount"] = self.maximum_member_count
        if self.roles is not None:
            properties["roles"] = [
                v.properties(
                )
                for v in self.roles
            ]
        if self.guild_master_role is not None:
            properties["guildMasterRole"] = self.guild_master_role
        if self.guild_member_default_role is not None:
            properties["guildMemberDefaultRole"] = self.guild_member_default_role
        if self.rejoin_cool_time_minutes is not None:
            properties["rejoinCoolTimeMinutes"] = self.rejoin_cool_time_minutes

        return properties



from __future__ import annotations
from abc import abstractmethod
from typing import List, Dict, Any

from .func import Func


class CdkResource:

    resource_name: str
    depends_on: List[str] = []

    def __init__(
            self,
            resource_name: str,
    ):
        self.depends_on = []
        self.resource_name = resource_name

    def default_resource_name(self):
        service_name = self.__module__.replace('gs2_cdk.', '').replace('.resource', '')
        return service_name[0].upper() + service_name[1:] + "_" + self.__class__.__name__ + "_" + self.alternate_keys()

    def add_depends_on(
            self,
            resource: CdkResource,
    ):
        self.depends_on.append(resource.resource_name)
        return self

    @abstractmethod
    def resource_type(self) -> str:
        raise NotImplementedError()

    @abstractmethod
    def properties(self) -> Dict[str, Any]:
        raise NotImplementedError(self)

    @abstractmethod
    def alternate_keys(self) -> str:
        raise NotImplementedError()

    def template(self) -> Dict[str, Any]:
        return {
            "Type": self.resource_type(),
            "Properties": self.properties(),
            "DependsOn": [
                depend
                for depend in self.depends_on
            ]
        }


class Stack:

    resources: List[CdkResource] = []
    outputs: Dict[str, str] = {}

    def __init__(self):
        self.resources = []
        self.outputs = {}

    def add_resource(
            self,
            resource: CdkResource,
    ):
        self.resources.append(resource)

    def output(
            self,
            name: str,
            path: str,
    ):
        self.outputs[name] = path

    def template(self) -> Dict[str, Any]:
        resources = {}
        for resource in self.resources:
            resources[resource.resource_name] = resource.template()
        outputs = {}
        for k, v in self.outputs.items():
            outputs[k] = v
        return {
            "GS2TemplateFormatVersion": "2019-05-01",
            "Resources": resources,
            "Outputs": outputs,
        }

    def yaml(self) -> str:
        import yaml

        def quoted_presenter(dumper, data):
            if len(data) > 0 and data[0] == '!':
                tag = data[:data.find(' ')]
                value = data[data.find(' ')+1:]
                if value[0] == '[':
                    return dumper.represent_sequence(tag, eval(value))
                else:
                    return dumper.represent_scalar(tag, value)
            return dumper.represent_scalar('tag:yaml.org,2002:str', data)

        def represent_str(dumper, instance):
            import re
            if "\n" in instance:
                instance = re.sub(' +\n| +$', '\n', instance)
                return dumper.represent_scalar('tag:yaml.org,2002:str', instance, style='|')
            else:
                return dumper.represent_scalar('tag:yaml.org,2002:str', instance)

        yaml.add_representer(str, quoted_presenter)
        yaml.add_representer(str, represent_str)

        return yaml.dump(self.template(), indent=2)


class TransactionSetting:

    distributor_namespace_id: str
    queue_namespace_id: str

    def __init__(
            self,
            distributor_namespace_id: str = None,
            queue_namespace_id: str = None,
    ):
        self.distributor_namespace_id = distributor_namespace_id
        self.queue_namespace_id = queue_namespace_id

    def properties(self) -> Dict[str, Any]:
        return {
            "EnableAutoRun": True,
            "DistributorNamespaceId": self.distributor_namespace_id,
            "QueueNamespaceId": self.queue_namespace_id,
        }


class ScriptSetting:

    trigger_script_id: str
    done_trigger_target_type: str
    done_trigger_script_id: str
    done_trigger_queue_namespace_id: str

    def __init__(
            self,
            trigger_script_id: str = None,
            done_trigger_target_type: str = 'none',
            done_trigger_script_id: str = None,
            done_trigger_queue_namespace_id: str = None,
    ):
        self.trigger_script_id = trigger_script_id
        self.done_trigger_target_type = done_trigger_target_type
        self.done_trigger_script_id = done_trigger_script_id
        self.done_trigger_queue_namespace_id = done_trigger_queue_namespace_id

    def properties(self) -> Dict[str, Any]:
        return {
            "TriggerScriptId": self.trigger_script_id,
            "DoneTriggerTargetType": self.done_trigger_target_type,
            "DoneTriggerScriptId": self.done_trigger_script_id,
            "DoneTriggerQueueNamespaceId": self.done_trigger_queue_namespace_id,
        }


class NotificationSetting:

    gateway_namespace_id: str
    enable_transfer_mobile_notification: bool
    sound: str

    def __init__(
            self,
            gateway_namespace_id: str = None,
            enable_transfer_mobile_notification: bool = None,
            sound: str = None,
    ):
        self.gateway_namespace_id = gateway_namespace_id
        self.enable_transfer_mobile_notification = enable_transfer_mobile_notification
        self.sound = sound

    def properties(self) -> Dict[str, Any]:
        return {
            "GatewayNamespaceId": self.gateway_namespace_id,
            "EnableTransferMobileNotification": self.enable_transfer_mobile_notification,
            "Sound": self.sound,
        }


class LogSetting:

    logging_namespace_id: str

    def __init__(
            self,
            logging_namespace_id: str,
    ):
        self.logging_namespace_id = logging_namespace_id

    def properties(self) -> Dict[str, Any]:
        return {
            "LoggingNamespaceId": self.logging_namespace_id,
        }


class Config:

    key: str
    value: str

    def __init__(
            self,
            key: str,
            value: str,
    ):
        self.key = key
        self.value = value

    def properties(self) -> Dict[str, Any]:
        return {
            "Key": self.key,
            "Value": self.value,
        }


class AcquireAction:

    action: str
    request: Dict[str, Any]

    def __init__(
            self,
            action: str,
            request: Dict[str, Any],
    ):
        self.action = action
        self.request = request

    def properties(self) -> Dict[str, Any]:
        properties = {}
        for k, v in self.request.items():
            properties[k] = v
        return {
            "action": self.action,
            "request": properties,
        }


class ConsumeAction:

    action: str
    request: Dict[str, Any]

    def __init__(
            self,
            action: str,
            request: Dict[str, Any],
    ):
        self.action = action
        self.request = request

    def properties(self) -> Dict[str, Any]:
        properties = {}
        for k, v in self.request.items():
            properties[k] = v
        return {
            "action": self.action,
            "request": properties,
        }



# Copyright 2016- Game Server Services, Inc. or its affiliates. All Rights
# Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License").
# You may not use this file except in compliance with the License.
# A copy of the License is located at
#
#  http://www.apache.org/licenses/LICENSE-2.0
#
# or in the "license" file accompanying this file. This file is distributed
# on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either
# express or implied. See the License for the specific language governing
# permissions and limitations under the License.
from .Namespace import Namespace
from .options.NamespaceOptions import NamespaceOptions
from .enum.NamespaceSupportSpeculativeExecution import NamespaceSupportSpeculativeExecution
from .options.NamespaceSupportSpeculativeExecutionIsEnableOptions import NamespaceSupportSpeculativeExecutionIsEnableOptions
from .options.NamespaceSupportSpeculativeExecutionIsDisableOptions import NamespaceSupportSpeculativeExecutionIsDisableOptions
from .StateMachineMaster import StateMachineMaster
from .options.StateMachineMasterOptions import StateMachineMasterOptions
from .StackEntry import StackEntry
from .options.StackEntryOptions import StackEntryOptions
from .Variable import Variable
from .options.VariableOptions import VariableOptions
from .Event import Event
from .options.EventOptions import EventOptions
from .enum.EventEventType import EventEventType
from .options.EventEventTypeIsChangeStateOptions import EventEventTypeIsChangeStateOptions
from .options.EventEventTypeIsEmitOptions import EventEventTypeIsEmitOptions
from .ChangeStateEvent import ChangeStateEvent
from .options.ChangeStateEventOptions import ChangeStateEventOptions
from .EmitEvent import EmitEvent
from .options.EmitEventOptions import EmitEventOptions
from .RandomStatus import RandomStatus
from .options.RandomStatusOptions import RandomStatusOptions
from .RandomUsed import RandomUsed
from .options.RandomUsedOptions import RandomUsedOptions


# Copyright 2016- Game Server Services, Inc. or its affiliates. All Rights
# Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License").
# You may not use this file except in compliance with the License.
# A copy of the License is located at
#
#  http://www.apache.org/licenses/LICENSE-2.0
#
# or in the "license" file accompanying this file. This file is distributed
# on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either
# express or implied. See the License for the specific language governing
# permissions and limitations under the License.
from .Namespace import Namespace
from .options.NamespaceOptions import NamespaceOptions
from .enum.NamespaceEnableDisconnectDetection import NamespaceEnableDisconnectDetection
from .enum.NamespaceCreateGatheringTriggerType import NamespaceCreateGatheringTriggerType
from .enum.NamespaceCompleteMatchmakingTriggerType import NamespaceCompleteMatchmakingTriggerType
from .enum.NamespaceEnableCollaborateSeasonRating import NamespaceEnableCollaborateSeasonRating
from .options.NamespaceEnableDisconnectDetectionIsDisableOptions import NamespaceEnableDisconnectDetectionIsDisableOptions
from .options.NamespaceEnableDisconnectDetectionIsEnableOptions import NamespaceEnableDisconnectDetectionIsEnableOptions
from .options.NamespaceCreateGatheringTriggerTypeIsNoneOptions import NamespaceCreateGatheringTriggerTypeIsNoneOptions
from .options.NamespaceCreateGatheringTriggerTypeIsGs2RealtimeOptions import NamespaceCreateGatheringTriggerTypeIsGs2RealtimeOptions
from .options.NamespaceCreateGatheringTriggerTypeIsGs2ScriptOptions import NamespaceCreateGatheringTriggerTypeIsGs2ScriptOptions
from .options.NamespaceCompleteMatchmakingTriggerTypeIsNoneOptions import NamespaceCompleteMatchmakingTriggerTypeIsNoneOptions
from .options.NamespaceCompleteMatchmakingTriggerTypeIsGs2RealtimeOptions import NamespaceCompleteMatchmakingTriggerTypeIsGs2RealtimeOptions
from .options.NamespaceCompleteMatchmakingTriggerTypeIsGs2ScriptOptions import NamespaceCompleteMatchmakingTriggerTypeIsGs2ScriptOptions
from .options.NamespaceEnableCollaborateSeasonRatingIsEnableOptions import NamespaceEnableCollaborateSeasonRatingIsEnableOptions
from .options.NamespaceEnableCollaborateSeasonRatingIsDisableOptions import NamespaceEnableCollaborateSeasonRatingIsDisableOptions
from .RatingModel import RatingModel
from .options.RatingModelOptions import RatingModelOptions
from .AttributeRange import AttributeRange
from .options.AttributeRangeOptions import AttributeRangeOptions
from .CapacityOfRole import CapacityOfRole
from .options.CapacityOfRoleOptions import CapacityOfRoleOptions
from .Attribute import Attribute
from .options.AttributeOptions import AttributeOptions
from .Player import Player
from .options.PlayerOptions import PlayerOptions
from .GameResult import GameResult
from .options.GameResultOptions import GameResultOptions
from .SignedBallot import SignedBallot
from .options.SignedBallotOptions import SignedBallotOptions
from .TimeSpan import TimeSpan
from .options.TimeSpanOptions import TimeSpanOptions
from .CurrentMasterData import CurrentMasterData


# Copyright 2016- Game Server Services, Inc. or its affiliates. All Rights
# Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License").
# You may not use this file except in compliance with the License.
# A copy of the License is located at
#
#  http://www.apache.org/licenses/LICENSE-2.0
#
# or in the "license" file accompanying this file. This file is distributed
# on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either
# express or implied. See the License for the specific language governing
# permissions and limitations under the License.
from __future__ import annotations
from typing import *
from .options.TargetCounterModelOptions import TargetCounterModelOptions
from .enum.TargetCounterModelResetType import TargetCounterModelResetType


class TargetCounterModel:
    counter_name: str
    value: int
    reset_type: Optional[TargetCounterModelResetType] = None

    def __init__(
        self,
        counter_name: str,
        value: int,
        options: Optional[TargetCounterModelOptions] = TargetCounterModelOptions(),
    ):
        self.counter_name = counter_name
        self.value = value
        self.reset_type = options.reset_type if options.reset_type else None

    def properties(
        self,
    ) -> Dict[str, Any]:
        properties: Dict[str, Any] = {}

        if self.counter_name is not None:
            properties["counterName"] = self.counter_name
        if self.reset_type is not None:
            properties["resetType"] = self.reset_type.value
        if self.value is not None:
            properties["value"] = self.value

        return properties



# Copyright 2016- Game Server Services, Inc. or its affiliates. All Rights
# Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License").
# You may not use this file except in compliance with the License.
# A copy of the License is located at
#
#  http://www.apache.org/licenses/LICENSE-2.0
#
# or in the "license" file accompanying this file. This file is distributed
# on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either
# express or implied. See the License for the specific language governing
# permissions and limitations under the License.
from __future__ import annotations
from typing import *
from ...core.model import AcquireAction
from .TimeSpan import TimeSpan
from .options.GlobalMessageOptions import GlobalMessageOptions


class GlobalMessage:
    name: str
    metadata: str
    read_acquire_actions: Optional[List[AcquireAction]] = None
    expires_time_span: Optional[TimeSpan] = None
    expires_at: Optional[int] = None
    message_reception_period_event_id: Optional[str] = None

    def __init__(
        self,
        name: str,
        metadata: str,
        options: Optional[GlobalMessageOptions] = GlobalMessageOptions(),
    ):
        self.name = name
        self.metadata = metadata
        self.read_acquire_actions = options.read_acquire_actions if options.read_acquire_actions else None
        self.expires_time_span = options.expires_time_span if options.expires_time_span else None
        self.expires_at = options.expires_at if options.expires_at else None
        self.message_reception_period_event_id = options.message_reception_period_event_id if options.message_reception_period_event_id else None

    def properties(
        self,
    ) -> Dict[str, Any]:
        properties: Dict[str, Any] = {}

        if self.name is not None:
            properties["name"] = self.name
        if self.metadata is not None:
            properties["metadata"] = self.metadata
        if self.read_acquire_actions is not None:
            properties["readAcquireActions"] = [
                v.properties(
                )
                for v in self.read_acquire_actions
            ]
        if self.expires_time_span is not None:
            properties["expiresTimeSpan"] = self.expires_time_span.properties(
            )
        if self.expires_at is not None:
            properties["expiresAt"] = self.expires_at
        if self.message_reception_period_event_id is not None:
            properties["messageReceptionPeriodEventId"] = self.message_reception_period_event_id

        return properties



# Copyright 2016- Game Server Services, Inc. or its affiliates. All Rights
# Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License").
# You may not use this file except in compliance with the License.
# A copy of the License is located at
#
#  http://www.apache.org/licenses/LICENSE-2.0
#
# or in the "license" file accompanying this file. This file is distributed
# on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either
# express or implied. See the License for the specific language governing
# permissions and limitations under the License.
from __future__ import annotations
from typing import *
from ...core.model import ConsumeAction
from ...core.model import AcquireAction
from .options.RandomDisplayItemModelOptions import RandomDisplayItemModelOptions


class RandomDisplayItemModel:
    name: str
    acquire_actions: List[AcquireAction]
    stock: int
    weight: int
    metadata: Optional[str] = None
    consume_actions: Optional[List[ConsumeAction]] = None

    def __init__(
        self,
        name: str,
        acquire_actions: List[AcquireAction],
        stock: int,
        weight: int,
        options: Optional[RandomDisplayItemModelOptions] = RandomDisplayItemModelOptions(),
    ):
        self.name = name
        self.acquire_actions = acquire_actions
        self.stock = stock
        self.weight = weight
        self.metadata = options.metadata if options.metadata else None
        self.consume_actions = options.consume_actions if options.consume_actions else None

    def properties(
        self,
    ) -> Dict[str, Any]:
        properties: Dict[str, Any] = {}

        if self.name is not None:
            properties["name"] = self.name
        if self.metadata is not None:
            properties["metadata"] = self.metadata
        if self.consume_actions is not None:
            properties["consumeActions"] = [
                v.properties(
                )
                for v in self.consume_actions
            ]
        if self.acquire_actions is not None:
            properties["acquireActions"] = [
                v.properties(
                )
                for v in self.acquire_actions
            ]
        if self.stock is not None:
            properties["stock"] = self.stock
        if self.weight is not None:
            properties["weight"] = self.weight

        return properties



# Copyright 2016- Game Server Services, Inc. or its affiliates. All Rights
# Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License").
# You may not use this file except in compliance with the License.
# A copy of the License is located at
#
#  http://www.apache.org/licenses/LICENSE-2.0
#
# or in the "license" file accompanying this file. This file is distributed
# on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either
# express or implied. See the License for the specific language governing
# permissions and limitations under the License.
from __future__ import annotations
from typing import *

from ...core.func import GetAttr, Join
from .InventoryModelRef import InventoryModelRef
from .SimpleInventoryModelRef import SimpleInventoryModelRef
from .BigInventoryModelRef import BigInventoryModelRef
from ..stamp_sheet.AddCapacityByUserId import AddCapacityByUserId
from ..stamp_sheet.SetCapacityByUserId import SetCapacityByUserId
from ..stamp_sheet.AcquireItemSetByUserId import AcquireItemSetByUserId
from ..stamp_sheet.AcquireItemSetWithGradeByUserId import AcquireItemSetWithGradeByUserId
from ..stamp_sheet.AddReferenceOfByUserId import AddReferenceOfByUserId
from ..stamp_sheet.DeleteReferenceOfByUserId import DeleteReferenceOfByUserId
from ..stamp_sheet.AcquireSimpleItemsByUserId import AcquireSimpleItemsByUserId
from ..model.AcquireCount import AcquireCount
from ..stamp_sheet.SetSimpleItemsByUserId import SetSimpleItemsByUserId
from ..stamp_sheet.AcquireBigItemByUserId import AcquireBigItemByUserId
from ..stamp_sheet.SetBigItemByUserId import SetBigItemByUserId
from ..stamp_sheet.VerifyInventoryCurrentMaxCapacityByUserId import VerifyInventoryCurrentMaxCapacityByUserId
from ..stamp_sheet.ConsumeItemSetByUserId import ConsumeItemSetByUserId
from ..stamp_sheet.VerifyItemSetByUserId import VerifyItemSetByUserId
from ..stamp_sheet.VerifyReferenceOfByUserId import VerifyReferenceOfByUserId
from ..stamp_sheet.ConsumeSimpleItemsByUserId import ConsumeSimpleItemsByUserId
from ..stamp_sheet.VerifySimpleItemByUserId import VerifySimpleItemByUserId
from ..stamp_sheet.ConsumeBigItemByUserId import ConsumeBigItemByUserId
from ..stamp_sheet.VerifyBigItemByUserId import VerifyBigItemByUserId


class NamespaceRef:
    namespace_name: str

    def __init__(
        self,
        namespace_name: str,
    ):
        self.namespace_name = namespace_name

    def inventory_model(
        self,
        inventory_name: str,
    ) -> InventoryModelRef:
        return InventoryModelRef(
            self.namespace_name,
            inventory_name,
        )

    def simple_inventory_model(
        self,
        inventory_name: str,
    ) -> SimpleInventoryModelRef:
        return SimpleInventoryModelRef(
            self.namespace_name,
            inventory_name,
        )

    def big_inventory_model(
        self,
        inventory_name: str,
    ) -> BigInventoryModelRef:
        return BigInventoryModelRef(
            self.namespace_name,
            inventory_name,
        )

    def add_capacity(
        self,
        inventory_name: str,
        add_capacity_value: int,
        user_id: Optional[str] = "#{userId}",
    ) -> AddCapacityByUserId:
        return AddCapacityByUserId(
            self.namespace_name,
            inventory_name,
            add_capacity_value,
            user_id,
        )

    def set_capacity(
        self,
        inventory_name: str,
        new_capacity_value: int,
        user_id: Optional[str] = "#{userId}",
    ) -> SetCapacityByUserId:
        return SetCapacityByUserId(
            self.namespace_name,
            inventory_name,
            new_capacity_value,
            user_id,
        )

    def acquire_item_set(
        self,
        inventory_name: str,
        item_name: str,
        acquire_count: int,
        expires_at: Optional[int] = None,
        create_new_item_set: Optional[bool] = None,
        item_set_name: Optional[str] = None,
        user_id: Optional[str] = "#{userId}",
    ) -> AcquireItemSetByUserId:
        return AcquireItemSetByUserId(
            self.namespace_name,
            inventory_name,
            item_name,
            acquire_count,
            expires_at,
            create_new_item_set,
            item_set_name,
            user_id,
        )

    def acquire_item_set_with_grade(
        self,
        inventory_name: str,
        item_name: str,
        grade_model_id: str,
        grade_value: int,
        user_id: Optional[str] = "#{userId}",
    ) -> AcquireItemSetWithGradeByUserId:
        return AcquireItemSetWithGradeByUserId(
            self.namespace_name,
            inventory_name,
            item_name,
            grade_model_id,
            grade_value,
            user_id,
        )

    def add_reference_of(
        self,
        inventory_name: str,
        item_name: str,
        reference_of: str,
        item_set_name: Optional[str] = None,
        user_id: Optional[str] = "#{userId}",
    ) -> AddReferenceOfByUserId:
        return AddReferenceOfByUserId(
            self.namespace_name,
            inventory_name,
            item_name,
            reference_of,
            item_set_name,
            user_id,
        )

    def delete_reference_of(
        self,
        inventory_name: str,
        item_name: str,
        reference_of: str,
        item_set_name: Optional[str] = None,
        user_id: Optional[str] = "#{userId}",
    ) -> DeleteReferenceOfByUserId:
        return DeleteReferenceOfByUserId(
            self.namespace_name,
            inventory_name,
            item_name,
            reference_of,
            item_set_name,
            user_id,
        )

    def acquire_simple_items(
        self,
        inventory_name: str,
        acquire_counts: List[AcquireCount],
        user_id: Optional[str] = "#{userId}",
    ) -> AcquireSimpleItemsByUserId:
        return AcquireSimpleItemsByUserId(
            self.namespace_name,
            inventory_name,
            acquire_counts,
            user_id,
        )

    def set_simple_items(
        self,
        inventory_name: str,
        counts: List[HeldCount],
        user_id: Optional[str] = "#{userId}",
    ) -> SetSimpleItemsByUserId:
        return SetSimpleItemsByUserId(
            self.namespace_name,
            inventory_name,
            counts,
            user_id,
        )

    def acquire_big_item(
        self,
        inventory_name: str,
        item_name: str,
        acquire_count: str,
        user_id: Optional[str] = "#{userId}",
    ) -> AcquireBigItemByUserId:
        return AcquireBigItemByUserId(
            self.namespace_name,
            inventory_name,
            item_name,
            acquire_count,
            user_id,
        )

    def set_big_item(
        self,
        inventory_name: str,
        item_name: str,
        count: str,
        user_id: Optional[str] = "#{userId}",
    ) -> SetBigItemByUserId:
        return SetBigItemByUserId(
            self.namespace_name,
            inventory_name,
            item_name,
            count,
            user_id,
        )

    def verify_inventory_current_max_capacity(
        self,
        inventory_name: str,
        verify_type: str,
        current_inventory_max_capacity: int,
        multiply_value_specifying_quantity: Optional[bool] = None,
        user_id: Optional[str] = "#{userId}",
    ) -> VerifyInventoryCurrentMaxCapacityByUserId:
        return VerifyInventoryCurrentMaxCapacityByUserId(
            self.namespace_name,
            inventory_name,
            verify_type,
            current_inventory_max_capacity,
            multiply_value_specifying_quantity,
            user_id,
        )

    def consume_item_set(
        self,
        inventory_name: str,
        item_name: str,
        consume_count: int,
        item_set_name: Optional[str] = None,
        user_id: Optional[str] = "#{userId}",
    ) -> ConsumeItemSetByUserId:
        return ConsumeItemSetByUserId(
            self.namespace_name,
            inventory_name,
            item_name,
            consume_count,
            item_set_name,
            user_id,
        )

    def verify_item_set(
        self,
        inventory_name: str,
        item_name: str,
        verify_type: str,
        count: int,
        item_set_name: Optional[str] = None,
        multiply_value_specifying_quantity: Optional[bool] = None,
        user_id: Optional[str] = "#{userId}",
    ) -> VerifyItemSetByUserId:
        return VerifyItemSetByUserId(
            self.namespace_name,
            inventory_name,
            item_name,
            verify_type,
            count,
            item_set_name,
            multiply_value_specifying_quantity,
            user_id,
        )

    def verify_reference_of(
        self,
        inventory_name: str,
        item_name: str,
        reference_of: str,
        verify_type: str,
        item_set_name: Optional[str] = None,
        user_id: Optional[str] = "#{userId}",
    ) -> VerifyReferenceOfByUserId:
        return VerifyReferenceOfByUserId(
            self.namespace_name,
            inventory_name,
            item_name,
            reference_of,
            verify_type,
            item_set_name,
            user_id,
        )

    def consume_simple_items(
        self,
        inventory_name: str,
        consume_counts: List[ConsumeCount],
        user_id: Optional[str] = "#{userId}",
    ) -> ConsumeSimpleItemsByUserId:
        return ConsumeSimpleItemsByUserId(
            self.namespace_name,
            inventory_name,
            consume_counts,
            user_id,
        )

    def verify_simple_item(
        self,
        inventory_name: str,
        item_name: str,
        verify_type: str,
        count: int,
        multiply_value_specifying_quantity: Optional[bool] = None,
        user_id: Optional[str] = "#{userId}",
    ) -> VerifySimpleItemByUserId:
        return VerifySimpleItemByUserId(
            self.namespace_name,
            inventory_name,
            item_name,
            verify_type,
            count,
            multiply_value_specifying_quantity,
            user_id,
        )

    def consume_big_item(
        self,
        inventory_name: str,
        item_name: str,
        consume_count: str,
        user_id: Optional[str] = "#{userId}",
    ) -> ConsumeBigItemByUserId:
        return ConsumeBigItemByUserId(
            self.namespace_name,
            inventory_name,
            item_name,
            consume_count,
            user_id,
        )

    def verify_big_item(
        self,
        inventory_name: str,
        item_name: str,
        verify_type: str,
        count: str,
        multiply_value_specifying_quantity: Optional[bool] = None,
        user_id: Optional[str] = "#{userId}",
    ) -> VerifyBigItemByUserId:
        return VerifyBigItemByUserId(
            self.namespace_name,
            inventory_name,
            item_name,
            verify_type,
            count,
            multiply_value_specifying_quantity,
            user_id,
        )

    def grn(
        self,
    ) -> str:
        return Join(
            ":",
            [
                "grn",
                "gs2",
                GetAttr.region(
                ).str(
                ),
                GetAttr.owner_id(
                ).str(
                ),
                "inventory",
                self.namespace_name,
            ],
        ).str(
        )



# Copyright 2016- Game Server Services, Inc. or its affiliates. All Rights
# Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License").
# You may not use this file except in compliance with the License.
# A copy of the License is located at
#
#  http://www.apache.org/licenses/LICENSE-2.0
#
# or in the "license" file accompanying this file. This file is distributed
# on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either
# express or implied. See the License for the specific language governing
# permissions and limitations under the License.
from __future__ import annotations
from typing import *
from .options.EmitEventOptions import EmitEventOptions


class EmitEvent:
    event: str
    parameters: str
    timestamp: int

    def __init__(
        self,
        event: str,
        parameters: str,
        timestamp: int,
        options: Optional[EmitEventOptions] = EmitEventOptions(),
    ):
        self.event = event
        self.parameters = parameters
        self.timestamp = timestamp

    def properties(
        self,
    ) -> Dict[str, Any]:
        properties: Dict[str, Any] = {}

        if self.event is not None:
            properties["event"] = self.event
        if self.parameters is not None:
            properties["parameters"] = self.parameters
        if self.timestamp is not None:
            properties["timestamp"] = self.timestamp

        return properties



# Copyright 2016- Game Server Services, Inc. or its affiliates. All Rights
# Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License").
# You may not use this file except in compliance with the License.
# A copy of the License is located at
#
#  http://www.apache.org/licenses/LICENSE-2.0
#
# or in the "license" file accompanying this file. This file is distributed
# on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either
# express or implied. See the License for the specific language governing
# permissions and limitations under the License.
from __future__ import annotations
from typing import *
from .options.TierModelOptions import TierModelOptions


class TierModel:
    raise_rank_bonus: int
    entry_fee: int
    minimum_change_point: int
    maximum_change_point: int
    metadata: Optional[str] = None

    def __init__(
        self,
        raise_rank_bonus: int,
        entry_fee: int,
        minimum_change_point: int,
        maximum_change_point: int,
        options: Optional[TierModelOptions] = TierModelOptions(),
    ):
        self.raise_rank_bonus = raise_rank_bonus
        self.entry_fee = entry_fee
        self.minimum_change_point = minimum_change_point
        self.maximum_change_point = maximum_change_point
        self.metadata = options.metadata if options.metadata else None

    def properties(
        self,
    ) -> Dict[str, Any]:
        properties: Dict[str, Any] = {}

        if self.metadata is not None:
            properties["metadata"] = self.metadata
        if self.raise_rank_bonus is not None:
            properties["raiseRankBonus"] = self.raise_rank_bonus
        if self.entry_fee is not None:
            properties["entryFee"] = self.entry_fee
        if self.minimum_change_point is not None:
            properties["minimumChangePoint"] = self.minimum_change_point
        if self.maximum_change_point is not None:
            properties["maximumChangePoint"] = self.maximum_change_point

        return properties



# Copyright 2016- Game Server Services, Inc. or its affiliates. All Rights
# Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License").
# You may not use this file except in compliance with the License.
# A copy of the License is located at
#
#  http://www.apache.org/licenses/LICENSE-2.0
#
# or in the "license" file accompanying this file. This file is distributed
# on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either
# express or implied. See the License for the specific language governing
# permissions and limitations under the License.
from __future__ import annotations
from typing import *
from .BalanceParameterValueModel import BalanceParameterValueModel
from .options.BalanceParameterModelOptions import BalanceParameterModelOptions
from .enum.BalanceParameterModelInitialValueStrategy import BalanceParameterModelInitialValueStrategy


class BalanceParameterModel:
    name: str
    total_value: int
    initial_value_strategy: BalanceParameterModelInitialValueStrategy
    parameters: List[BalanceParameterValueModel]
    metadata: Optional[str] = None

    def __init__(
        self,
        name: str,
        total_value: int,
        initial_value_strategy: BalanceParameterModelInitialValueStrategy,
        parameters: List[BalanceParameterValueModel],
        options: Optional[BalanceParameterModelOptions] = BalanceParameterModelOptions(),
    ):
        self.name = name
        self.total_value = total_value
        self.initial_value_strategy = initial_value_strategy
        self.parameters = parameters
        self.metadata = options.metadata if options.metadata else None

    def properties(
        self,
    ) -> Dict[str, Any]:
        properties: Dict[str, Any] = {}

        if self.name is not None:
            properties["name"] = self.name
        if self.metadata is not None:
            properties["metadata"] = self.metadata
        if self.total_value is not None:
            properties["totalValue"] = self.total_value
        if self.initial_value_strategy is not None:
            properties["initialValueStrategy"] = self.initial_value_strategy.value
        if self.parameters is not None:
            properties["parameters"] = [
                v.properties(
                )
                for v in self.parameters
            ]

        return properties



# Copyright 2016- Game Server Services, Inc. or its affiliates. All Rights
# Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License").
# You may not use this file except in compliance with the License.
# A copy of the License is located at
#
#  http://www.apache.org/licenses/LICENSE-2.0
#
# or in the "license" file accompanying this file. This file is distributed
# on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either
# express or implied. See the License for the specific language governing
# permissions and limitations under the License.
from __future__ import annotations
from typing import *

from ...core.model import CdkResource, Stack
from ...core.func import GetAttr
from ...core.model import ScriptSetting
from ...core.model import NotificationSetting
from ...core.model import LogSetting

from ..ref.NamespaceRef import NamespaceRef

from .options.NamespaceOptions import NamespaceOptions


class Namespace(CdkResource):
    stack: Stack
    name: str
    description: Optional[str] = None
    allow_create_room: Optional[bool] = None
    post_message_script: Optional[ScriptSetting] = None
    create_room_script: Optional[ScriptSetting] = None
    delete_room_script: Optional[ScriptSetting] = None
    subscribe_room_script: Optional[ScriptSetting] = None
    unsubscribe_room_script: Optional[ScriptSetting] = None
    post_notification: Optional[NotificationSetting] = None
    log_setting: Optional[LogSetting] = None

    def __init__(
        self,
        stack: Stack,
        name: str,
        options: Optional[NamespaceOptions] = NamespaceOptions(),
    ):
        super().__init__(
            "Chat_Namespace_" + name
        )

        self.stack = stack
        self.name = name
        self.description = options.description if options.description else None
        self.allow_create_room = options.allow_create_room if options.allow_create_room else None
        self.post_message_script = options.post_message_script if options.post_message_script else None
        self.create_room_script = options.create_room_script if options.create_room_script else None
        self.delete_room_script = options.delete_room_script if options.delete_room_script else None
        self.subscribe_room_script = options.subscribe_room_script if options.subscribe_room_script else None
        self.unsubscribe_room_script = options.unsubscribe_room_script if options.unsubscribe_room_script else None
        self.post_notification = options.post_notification if options.post_notification else None
        self.log_setting = options.log_setting if options.log_setting else None
        stack.add_resource(
            self,
        )


    def alternate_keys(
        self,
    ):
        return "name"

    def resource_type(
        self,
    ) -> str:
        return "GS2::Chat::Namespace"

    def properties(
        self,
    ) -> Dict[str, Any]:
        properties: Dict[str, Any] = {}

        if self.name is not None:
            properties["Name"] = self.name
        if self.description is not None:
            properties["Description"] = self.description
        if self.allow_create_room is not None:
            properties["AllowCreateRoom"] = self.allow_create_room
        if self.post_message_script is not None:
            properties["PostMessageScript"] = self.post_message_script.properties(
            )
        if self.create_room_script is not None:
            properties["CreateRoomScript"] = self.create_room_script.properties(
            )
        if self.delete_room_script is not None:
            properties["DeleteRoomScript"] = self.delete_room_script.properties(
            )
        if self.subscribe_room_script is not None:
            properties["SubscribeRoomScript"] = self.subscribe_room_script.properties(
            )
        if self.unsubscribe_room_script is not None:
            properties["UnsubscribeRoomScript"] = self.unsubscribe_room_script.properties(
            )
        if self.post_notification is not None:
            properties["PostNotification"] = self.post_notification.properties(
            )
        if self.log_setting is not None:
            properties["LogSetting"] = self.log_setting.properties(
            )

        return properties

    def ref(
        self,
    ) -> NamespaceRef:
        return NamespaceRef(
            self.name,
        )

    def get_attr_namespace_id(
        self,
    ) -> GetAttr:
        return GetAttr(
            self,
            "Item.NamespaceId",
            None,
        )
